@@ -1,5 +1,5 @@
 #!/usr/bin/Env python
-# encoding: utf-8
+# -*- coding: utf-8 -*-
 
 """Community Codeswarm - create a shared codeswarm for multiple projects."""
 
@@ -9,6 +9,9 @@ Copyright © 2008 Arne Babenhauserheide
 License: GPLv3
 """
 
+# TODO: Ignore changesets with the same ID. 
+
+
 # yaml for parsing the config file - somehow overkill, but very easy to use.
 from yaml import load
 # Changing directories, cause I don't want to join paths. 
@@ -30,6 +33,8 @@ from subprocess import call
 from sys import argv
 # Mercurial for using PyMarkdownMinisite - committing the change to the website file and pushing the data. 
 from mercurial import dispatch
+# also we need the color range for choosing colors automatically. 
+from color_range import get_color_string
 
 ### Constants ###
 
@@ -97,7 +102,7 @@ for i in [INCOMING_REPO_DIR, PROCESSED_REPO_DIR]:
 current_dir = abspath(curdir)
 chdir(INCOMING_REPO_DIR)
 for i in simple_config.values(): 
-    print i
+    print "\n" + str(i)
     call(i, shell=True)
 
 # Now change back into the main dir and check incoming for all repoisitories. 
@@ -105,18 +110,21 @@ chdir(current_dir)
 # First empty shared.log
 call("rm shared.log", shell=True)
 
+# Now init a list of known node IDs, so we never count any changeset twice. 
+known_nodes = []
+
 for repo in simple_config: 
     incoming_path = join(INCOMING_REPO_DIR, repo)
     target_path = join(PROCESSED_REPO_DIR, repo)
     # The log goeds into a temporary file first. We need more that simple sed. 
     # If there's already a repository, we do an incoming check. 
     if isdir(target_path): 
-	call("hg incoming --style=code_swarm -q -R " + target_path + " " + incoming_path + " > tmp.log", shell=True)
+	call("hg incoming --style=code_swarm_ext -q -R " + target_path + " " + incoming_path + " > tmp.log", shell=True)
 	# now we pull it. 
 	call("hg pull -R " + target_path + " " + incoming_path, shell=True)
     # Else we get the log from the incoming repository and then clone it. 
     else: 
-	call("hg log --style=code_swarm -q -R " + incoming_path + " > tmp.log", shell=True)
+	call("hg log --style=code_swarm_ext -q -R " + incoming_path + " > tmp.log", shell=True)
 	call("hg clone -U " + incoming_path + " " + target_path, shell=True)
 
     # Now convert the code_swarm styled temp file. 
@@ -139,18 +147,24 @@ for repo in simple_config:
 	else: # If the line is empty, we add a new sublist. 
 	    tmp_nested.append([])
     
-    # Now we need to change all files for the entries entries to begin with the repository name as path - we keep the authors, as they might be active across different projects. 
-    for entry in tmp_nested: 
+    # Now we need to change all files for the entries entries to begin with the repository name as path - we keep the node IDs (which are unique) and the authors, as they might be active across different projects, as well as the date. 
+    for entry in tmp_nested:
 	# Only change the files. 
-	for i in range(len(entry[2:])): 
-	    entry[i+2] = join(repo, entry[i+2])
+	for i in range(len(entry[3:])): 
+	    entry[i+3] = join(repo, entry[i+3])
 
-    # After this we need to unnest the list. 
+    # After this we need to unnest the list
+    # and make sure we only add each changeset once. 
     temp_lines = []
     for entry in tmp_nested: 
-	# Append all lines in the entry. 
-	for line in entry: 
-	    temp_lines.append(line)
+        # If the node has already been added, ignore this changeset. 
+        if entry and entry[0] in known_nodes: 
+            continue
+        elif entry: 
+            known_nodes.append(entry[0])
+	# Append all lines after the first (the node ID) in the entry. 
+	for i in range(len(entry)-1): 
+	    temp_lines.append(entry[i+1])
 	# Also append an empty line as entry seperator. 
 	temp_lines.append("")
 
@@ -161,6 +175,9 @@ for repo in simple_config:
     
     # Done - we have a shared log. 
 
+# We can now safely delete the known_nodes list, since we don't need it anymore. 
+del known_nodes
+
 ## Step 2: Turn the log into a codeswarm
 
 # Only go on if the log contains something. If it doesn't, we can stop here. 
@@ -182,7 +199,7 @@ call("python " + join(SCRIPT_DIR, "convert_logs_hg.py") + " -m shared.log", shel
 codeswarm_data_dir = join(CODESWARM_PROJECT_DIR, "data")
 call("cp shared.xml " + codeswarm_data_dir, shell=True)
 
-# Also add a config. TODO: auto generate a config for the different projects with different colors - maybe with also matching to .*doc.* and .*test.*
+# Also add a config. 
 call("cp " + join(SCRIPT_DIR, CONFIG_FOR_CODESWARM) + " " + codeswarm_data_dir, shell=True)
 
 # Now change into the codeswarm dir. 
@@ -190,8 +207,13 @@ chdir(CODESWARM_PROJECT_DIR)
 # and start the codeswarm - If we have an adapted codeswarm config, we use that. 
 if isfile(join(current_dir, CONFIG_FOR_CODESWARM)): 
     call("./run.sh" + " " + join(current_dir, CONFIG_FOR_CODESWARM),  shell=True)
-else: # we use the one from the shared codeswarm repository. 
-    call("./run.sh" + " " + join(SCRIPT_DIR, CONFIG_FOR_CODESWARM),  shell=True)
+else: # we use the one from the shared codeswarm repository and add a color string. 
+    # for this we copy the config and add custom color lines
+    conf = CONFIG_FOR_CODESWARM + "_tmp"
+    call("cat " + join(SCRIPT_DIR, CONFIG_FOR_CODESWARM) + " > " + join(SCRIPT_DIR, conf), shell=True)
+    color_lines = get_color_string(simple_config.keys())
+    call("echo '\n" + color_lines + "' >> " + join(SCRIPT_DIR, conf), shell=True)
+    call("./run.sh" + " " + join(SCRIPT_DIR, conf),  shell=True)
 
 
 ## Step 3: Create the video - yes, I am lazy :) . 
